/***************************************************************************************
*
* Conversion examples.asm
*
* Accompaniment for:
* Guide for Porting Code from SHARC DSPs to TigerSHARC Processors (EE-241)
*
* Written by Andrew Caldwell and Maikel Kokaly-Bannourah 
*	
* June 2004 rev 1.0 
*
* This file shows a one-to-one conversion of the SHARC code, provided in the
* SHARC to TigerSHARC Conversion Examples section, to TigerSHARC. 
*
***************************************************************************************/

#define N 8

#ifdef __ADSPTS201__
	#include <defts201.h>
#endif

#ifdef __ADSPTS101__
	#include <defts101.h>
#endif
/*************************************************************************/

.global _main;

.section data1;
.align N;
.VAR input1[N] = 0,1,2,3,4,5,6,7;
.VAR input2[N] = 0,1,2,3,4,5,6,7;
.VAR output[N];

.align 4;
.VAR buffer[N] = 0xa, 0xB, 0xc, 0xd,0xe,0xf,0x0,0x1;

.section program;

//-------------Start of code--------------------------------------------------
_main:


// REGISTER FILE

//	PEx vs. CBx
	
	//r0 = 0x1234;						
	xr0 = 0x1234;;
	//r1 = 0x5678;						
	xr1 = 0x5678;;

//	PEx (Secondary Regs.) vs CBx (NO secondary registers)

	//Bit set MODE1 SRRFL;		// xr31-16 are used 
	//nop;				// No need to enable mode
	//r0 = 0x1234;
	xr16 = 0x1234;;
	//r1 = 0x5678;						
	xr17 = 0x5678;;

	
// DATA ADDRESSING

//Setting up registers
	//r0 = 0x1234;
	xr0 = 0x1234;;
	//r1 = 0x5678;		
	xr1 = 0x5678;;
	//i0 = input1;		
	j4 = input1;;
	//m0 = 1;						
	j12 = 1;;
	//i8 = input2;
	k4 = input2;;
	//m8 = 1;
	k12 = 1;;

// Post-modify DAG1 operation			
	//dm(i0, m0)= r0;					
//Post-modify JALU operation
	[j4 +=j12]=xr0;;

// Pre-modify DAG2 operation		
	//pm(m8, i8)= r1;					
//Pre-modify KALU operation
	[k4 + k12]=xr1;;	

// Parallel DAG1 & DAG2 STORE			
	//r0=dm(i0, m0), r1=pm(i8, m8);			
// Parallel JALU & KALU STORE	
	xr0=[j4 +=j12]; xr1=[k4 += k12];;	
								
// Parallel DAG1 & DAG2 LOAD			
	//dm(i0, m0)= r0, pm(i8, m8)= r1;		
// Parallel JALU & KALU LOAD		
	[j4 +=j12]=xr0;;
	[k4 +=k12]=xr1;;	
	

// CIRCULAR BUFFERS

// bit set mode1 CBUFEN;				// Enabled in TigerSHARC by
// Enable circular mode (2116x only)	// using "CB"

	//r1 = 0x1234;				
	xr1 = 0x1234;;
	//r2 = 0x5678;
	xr2 = 0x5678;;	
	
	//B0 = output;	// Base for buffer circular		
	jB0 = output;;	// Base for buffer
	//I0 = b0;		// initialize index		
	j0 = jB0;;		// initialize index
	//L0 = 3;		// length circular		
	jL0 = 3;;		// Length of buffer
	//M0 = 1;		// different modifier		
	j12 = 1;;		// Modifier
	
	//dm(i0,m0)=r1;	// i0=buffer(top)		
	CB[j0+=j12] = xr1;;	// j0=buffer(top)
	//dm(i0,m0)=r1;	// i0=buffer+1			
	CB[j0+=j12] = xr1;;	// j0=buffer+1
	//dm(i0,m0)=r1;	// i0=buffer+2			
	CB[j0+=j12] = xr1;;	// j0=buffer+2
	//dm(i0,m0)=r2;	// i0=buffer(top)		
	CB[j0+=j12] = xr2;;	// j0=buffer(top)

		
// SISD, SIMD and Broadcast Loading

	//I1=buffer; 
	j5=buffer;;
	//M1=2;	
	j13=2;;
	
	//SISD			
	//r0=dm(i1,0);	//r0 loaded with 0xA	
	xr0=[j5+j31];;	//xr0 loaded with 0xA
	
	//SIMD							
	//Bit Set MODE1 PEYEN;				//SIMD not a mode ->no need to enable
	//nop;								//Specified by register names
	//r1=dm(i1,m1);		//Load r1 with 0xA		//Load s1 with 0xB	
	yxr1=L[j5+=j13];;	//Load xr1 with 0xA		//Load yr1 with 0xB
								
	
	//Broadcast Loading Mode			
	//Bit Set MODE1 BDCST1;				//Broadcast not a mode ->no need to enable
	//nop;							//Specified by register names
	//r2=dm(i1,m1);	//Load r2 with 0xC	//Load s2 with 0xC	
	r2=[j5+=j13];;	//Load xr2 with 0xC	//Load yr2 with 0xC

	nop;;nop;;nop;;nop;;

wait0: jump wait0;;	

_main.end:



